import {
  Controller,
  Get,
  Post,
  Body,
  Patch,
  Param,
  Delete,
  NotFoundException,
  HttpCode,
  BadRequestException,
} from '@nestjs/common';
import { ApiResponse } from '@nestjs/swagger';
import { ContractsService } from './contracts.service';
import { Contract } from './entities/contract.entity';
import { ContractCreateDto } from './dto/contract-create.dto';
import { ContractUpdateDto } from './dto/contract-update.dto';
import { EntityNotFoundError } from 'typeorm';
import { Paginate, PaginateQuery, Paginated } from 'nestjs-paginate';

@Controller()
export class ContractsController {
  constructor(private readonly contractsService: ContractsService) {}

  private handleEntityNotFoundError(id: number, e: Error) {
    if (e instanceof EntityNotFoundError) {
      throw new NotFoundException(`Not found: id=${id}`);
    } else {
      throw e;
    }
  }

  @Get()
  @ApiResponse({ type: [Contract] })
  findAll(@Paginate() query: PaginateQuery): Promise<Paginated<Contract>> {
    return this.contractsService.findAll(query);
  }

  @Get(':id')
  @ApiResponse({ type: Contract })
  async findOne(@Param('id') id: number): Promise<Contract | null> {
    // NOTE: the + operator returns the numeric representation of the object.
    return this.contractsService.findOne(+id).catch((err) => {
      this.handleEntityNotFoundError(id, err);
      return null;
    });
  }

  @Post()
  @ApiResponse({ type: [Contract] })
  create(@Body() contractCreateDto: ContractCreateDto) {
    return this.contractsService.create(contractCreateDto);
  }

  @Patch(':id')
  // Status 204 because no content will be returned
  @HttpCode(204)
  async update(
    @Param('id') id: number,
    @Body() contractUpdateDto: ContractUpdateDto,
  ): Promise<void> {
    if (Object.keys(contractUpdateDto).length === 0) {
      throw new BadRequestException('Request body is empty');
    }
    await this.contractsService.update(+id, contractUpdateDto).catch((err) => {
      this.handleEntityNotFoundError(id, err);
    });
  }

  @Delete(':id')
  @HttpCode(204)
  async remove(@Param('id') id: number): Promise<void> {
    await this.contractsService.remove(+id).catch((err) => {
      this.handleEntityNotFoundError(id, err);
    });
  }
}
