import { Injectable, Logger } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { EntityNotFoundError, Repository } from 'typeorm';
import { Contract } from './entities/contract.entity';
import { ContractCreateDto } from './dto/contract-create.dto';
import { ContractUpdateDto } from './dto/contract-update.dto';
import {
  FilterOperator,
  FilterSuffix,
  PaginateQuery,
  Paginated,
  paginate,
} from 'nestjs-paginate';

@Injectable()
export class ContractsService {
  constructor(
    private readonly logger: Logger,
    @InjectRepository(Contract)
    private contractsRepository: Repository<Contract>,
  ) {}

  create(contractCreateDto: ContractCreateDto): Promise<Contract> {
    // Use Repository.create() will copy the values to destination object.
    const contract = this.contractsRepository.create(contractCreateDto);
    return this.contractsRepository.save(contract);
  }

  findAll(query: PaginateQuery): Promise<Paginated<Contract>> {
    return paginate(query, this.contractsRepository, {
      relations: { staff: true },
      sortableColumns: ['id', 'title', 'details', 'staff.name'],
      select: ['id', 'title', 'details', 'staff.name'],
      filterableColumns: {
        title: [FilterOperator.EQ, FilterOperator.ILIKE, FilterSuffix.NOT],
        // staff.name: [FilterOperator.EQ, FilterOperator.ILIKE, FilterSuffix.NOT],
      },
    });
  }

  findOne(id: number): Promise<Contract> {
    return this.contractsRepository.findOneOrFail({ where: { id } });
  }

  async update(
    id: number,
    contractUpdateDto: ContractUpdateDto,
  ): Promise<void> {
    await this.isExist(id);
    await this.contractsRepository.update({ id }, contractUpdateDto);
  }

  async remove(id: number): Promise<void> {
    await this.isExist(id);
    await this.contractsRepository.delete(id);
  }

  // Helper function: Check if the entity exist.
  // If entity does not exsit, return the Promise.reject()
  private async isExist(id: number): Promise<void> {
    const cnt = await this.contractsRepository.countBy({ id });
    if (cnt > 0) {
      return;
    } else {
      return Promise.reject(
        new EntityNotFoundError(Contract, { where: { id } }),
      );
    }
  }
}
